<!DOCTYPE html>
<h1>Hello :)</h1>

<p>User: {{ user }}</p>

{% if bar %}
<ul>
{% for bar in foo %}
    <li>{{ object }}</li>
{% endfor %}
</ul>
{% endif %}

<p>Do we want spam? {{ spam|yesno }}</p>
<p>привет-привет.</p>
